import { declare } from '@babel/helper-plugin-utils';
import { types } from '@babel/core';

function pushAccessor(mutatorMap, node) {
  const alias = types.toKeyAlias(node);
  const map = mutatorMap[alias] ??= {
    _inherits: [],
    _key: node.key
  };
  map._inherits.push(node);
  const value = types.functionExpression(null, node.params, node.body, node.generator, node.async);
  value.returnType = node.returnType;
  types.inheritsComments(value, node);
  map[node.kind] = value;
  return map;
}
function toDefineObject(mutatorMap) {
  const objExpr = types.objectExpression([]);
  Object.keys(mutatorMap).forEach(function (mutatorMapKey) {
    const map = mutatorMap[mutatorMapKey];
    map.configurable = types.booleanLiteral(true);
    map.enumerable = types.booleanLiteral(true);
    const mapNode = types.objectExpression([]);
    const propNode = types.objectProperty(map._key, mapNode, map._computed);
    Object.keys(map).forEach(function (key) {
      const node = map[key];
      if (key[0] === "_") return;
      const prop = types.objectProperty(types.identifier(key), node);
      types.inheritsComments(prop, node);
      types.removeComments(node);
      mapNode.properties.push(prop);
    });
    objExpr.properties.push(propNode);
  });
  return objExpr;
}

var index = declare(api => {
  api.assertVersion("8.0.0-alpha.5");
  return {
    name: "transform-property-mutators",
    visitor: {
      ObjectExpression(path) {
        const {
          node
        } = path;
        let mutatorMap;
        const newProperties = node.properties.filter(function (prop) {
          if (types.isObjectMethod(prop) && !prop.computed && (prop.kind === "get" || prop.kind === "set")) {
            pushAccessor(mutatorMap ??= {}, prop);
            return false;
          }
          return true;
        });
        if (mutatorMap === undefined) {
          return;
        }
        node.properties = newProperties;
        path.replaceWith(types.callExpression(types.memberExpression(types.identifier("Object"), types.identifier("defineProperties")), [node, toDefineObject(mutatorMap)]));
      }
    }
  };
});

export { index as default };
//# sourceMappingURL=index.js.map
